import {Component, ElementRef, OnInit, ViewChild} from '@angular/core';
import {MatButton} from '@angular/material/button';
import {PatientContextService} from '../services/patient-context.service';

@Component({
  selector: 'app-cpr-search',
  templateUrl: './cpr-search.component.html',
  styleUrls: ['./cpr-search.component.scss']
})
export class CprSearchComponent implements OnInit {
  @ViewChild('wrapper') wrapper?: ElementRef<HTMLElement>;
  @ViewChild('submitButton') submitButton?: MatButton;
  public cpr = '';

  constructor(private patientContext: PatientContextService) { }

  ngOnInit(): void {
    this.patientContext.cpr$.subscribe(() => {
      this.cpr = '';
    });
  }

  public search(cpr: string): void {
    this.patientContext.setCpr(cpr);
  }

  public handleInputFocus(): void {
    this.wrapper?.nativeElement.classList.add('focused');
  }

  public handleInputBlur(): void {
    this.wrapper?.nativeElement.classList.remove('focused');
  }

  public handleEnterPressed(): { andSearch: (cpr: string) => void } {
    this.submitButton?.ripple.launch({centered: true});
    return {
      andSearch: (cpr) => this.search(cpr)
    };
  }
}
