import {fakeAsync, flush, TestBed, tick} from '@angular/core/testing';

import { DataSourceService } from './data-source.service';

import {CarePlanService, DataCardResponse, DataCardService, ObservationService} from 'facade-api';
import {of, Subject} from 'rxjs';
import {HttpClientTestingModule} from '@angular/common/http/testing';
import {PatientContextService} from './patient-context.service';

describe('DataSourceService', () => {
  let service: DataSourceService;
  let dataCardService: DataCardService;
  let episodeOfCare = new Subject<string | undefined>();

  beforeEach(async () => {
    episodeOfCare = new Subject<string | undefined>();
    dataCardService = {
      maternityDataCardPut: (id: string, version: string) => of({documentUniqueId: id, version: String(Number(version) + 1)} as any),
      maternityDataCardGetAll: () => of([
        {documentUniqueId: '1234', version: '5', episodeOfCareIdentifier: '1'} as any,
        {documentUniqueId: '789', version: '3', episodeOfCareIdentifier: '2'} as any,
      ]),
    } as any;

    await TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [
        {provide: DataCardService, useValue: dataCardService},
        {provide: CarePlanService, useValue: {}},
        {provide: ObservationService, useValue: {}},
        {provide: PatientContextService, useValue: {getCurrentEpisodeOfCare: () => '1', episodeOfCare$: episodeOfCare}},
      ]
    }).compileComponents();
    service = TestBed.inject(DataSourceService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('updates activeDataCard$ observable after saving datacard', fakeAsync(() => {
    const invocations: Array<DataCardResponse|undefined> = [];
    service.activeDataCard$.subscribe((card) => {
      invocations.push(card);
    });
    service.updateDataCard('123', '1', {} as any).toPromise();
    flush();
    expect(invocations.length).toBeGreaterThanOrEqual(1);
    const card = invocations[invocations.length - 1];
    expect(card).toBeDefined();
    expect(card?.documentUniqueId).toBe('123');
    expect(card?.version).toBe('2');
  }));

  it('updates activeDataCard$ when asked to refresh', fakeAsync(() => {
    let called = false;
    service.activeDataCard$.subscribe((card) => {
      expect(card).toBeDefined();
      expect(card?.documentUniqueId).toBe('1234');
      expect(card?.version).toBe('5');
      called = true;
    });
    service.refresh();
    flush();
    expect(called).toBeTrue();
  }));

  it('updates activeDataCard$ episodeOfCare$ changes', fakeAsync(() => {
    const invocations: Array<DataCardResponse|undefined> = [];
    service.activeDataCard$.subscribe((card) => {
        invocations.push(card);
    });
    episodeOfCare.next('2');
    flush();
    expect(invocations.length).toBeGreaterThanOrEqual(1);
    const card = invocations[invocations.length - 1];
    expect(card).toBeDefined();
    expect(card?.documentUniqueId).toBe('789');
    expect(card?.version).toBe('3');
  }));
});
