import {fakeAsync, flush, TestBed} from '@angular/core/testing';

import {PatientContextService} from './patient-context.service';
import {WindowService} from './window.service';

describe('PatientContextService', () => {
  let service: PatientContextService;

  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [
        {provider: WindowService, useValue: {nativeWindow: () => ({
              document: {cookie: ''},
              localStorage: {getItem: () => null, setItem: () => undefined}}
            )}
        }
      ]
    });
    service = TestBed.inject(PatientContextService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('triggers observable when patient changes', fakeAsync(() => {
    let calledTimes = 0;
    const sub = service.cpr$.subscribe((cpr) => {
      expect(cpr).toBe('154789542');
      calledTimes++;
    });
    service.setCpr('154789542');
    flush();
    expect(calledTimes).toBe(1);
    sub.unsubscribe();
  }));

  it('clears episodeOfCare when patient changes', fakeAsync(() => {
    let calledTimes = 0;
    service.setCpr('1234567890');
    service.setEpisodeOfCare('3');
    flush();
    const sub = service.episodeOfCare$.subscribe((episode) => {
      expect(episode).toBeUndefined();
      calledTimes++;
    });
    service.setCpr('0987654321');
    flush();
    expect(calledTimes).toBe(1);
    sub.unsubscribe();
  }));

  it('gets patient synchronously', () => {
    service.setCpr('1234567890');
    expect(service.getCurrentCpr()).toBe('1234567890');
  });

  it('triggers observable when episodeOfCare changes', fakeAsync(() => {
    let calledTimes = 0;
    const sub = service.episodeOfCare$.subscribe((episode) => {
      expect(episode).toBe('4');
      calledTimes++;
    });
    service.setEpisodeOfCare('4');
    flush();
    expect(calledTimes).toBe(1);
    sub.unsubscribe();
  }));

  it('gets episodeOfCare synchronously', () => {
    service.setEpisodeOfCare('77');
    expect(service.getCurrentEpisodeOfCare()).toBe('77');
  });
});
